/**
 * ----------------------------------------------------------------------------------
 *
 * Copyright (C) 2011 runlevel3 GmbH Berlin
 *
 * ----------------------------------------------------------------------------------
 *
 * LICENSE:
 *
 * This file is part of USB-AD-Interface.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * ----------------------------------------------------------------------------------
 *
 * AVR Studio 4.18 / avr-gcc version 4.3.3
 *
 * ----------------------------------------------------------------------------------
 *
 * @copyright 2011 runlevel3 GmbH
 * @license   http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @link      http://www.runlevel3.de runlevel3
 **/


#ifndef ad7797_h_
#define ad7797_h_

//hardware configuration
#define	AD7797_SPI_PORT		PORTB
#define	AD7797_SPI_PIN		PINB
#define	AD7797_SPI_DDR		DDRB
#define	SPI_BIT_SS			4
#define	SPI_BIT_SCK			7
#define	SPI_BIT_MOSI		5
#define	SPI_BIT_MISO		6


//chip IDs
#define	AD7796_ID				0x5A
#define	AD7797_ID				0x5B

//status register bits
#define	AD7797_STAT_RDY			0x80
#define	AD7797_STAT_ERR			0x40
#define	AD7797_STAT_CH2			0x04
#define	AD7797_STAT_CH1			0x02
#define	AD7797_STAT_CH0			0x01

//communication register bits
#define	AD7797_COM_WEN			0x80
#define	AD7797_COM_READ			0x40
#define	AD7797_COM_RS2			0x20
#define	AD7797_COM_RS1			0x10
#define	AD7797_COM_RS0			0x08
#define	AD7797_COM_CREAD		0x04
//register select definitions
#define	AD7797_RS_COM			0x00
#define	AD7797_RS_COM_SIZE		1
#define	AD7797_RS_MODE			AD7797_COM_RS0
#define	AD7797_RS_MODE_SIZE		2
#define	AD7797_RS_CONF			AD7797_COM_RS1
#define	AD7797_RS_CONF_SIZE		2
#define	AD7797_RS_DATA			AD7797_COM_RS0 | AD7797_COM_RS1
#define	AD7797_RS_DATA_SIZE		3
#define	AD7797_RS_ID			AD7797_COM_RS2
#define	AD7797_RS_ID_SIZE		1
#define	AD7797_RS_RES			AD7797_COM_RS0 | AD7797_COM_RS2
#define	AD7797_RS_RES_SIZE		1
#define	AD7797_RS_OFFSET		AD7797_COM_RS1 | AD7797_COM_RS2
#define	AD7797_RS_OFFSET_SIZE	3
#define	AD7797_RS_FS			AD7797_COM_RS0 | AD7797_COM_RS1 | AD7797_COM_RS2
#define	AD7797_RS_FS_SIZE		3

//mode register bits
#define	AD7797_MODE_MD2			0x8000
#define	AD7797_MODE_MD1			0x4000
#define	AD7797_MODE_MD0			0x2000
#define	AD7797_MODE_CLK1		0x0080
#define	AD7797_MODE_CLK0		0x0040
#define	AD7797_MODE_FS3			0x0008
#define	AD7797_MODE_FS2			0x0004
#define	AD7797_MODE_FS1			0x0002
#define	AD7797_MODE_FS0			0x0001
//mode register definitions
#define	AD7797_MODE_CONTINUOUS	0x0000
#define	AD7797_MODE_SINGLE		AD7797_MODE_MD0
#define	AD7797_MODE_IDLE		AD7797_MODE_MD1
#define	AD7797_MODE_POWERDOWN	AD7797_MODE_MD0 | AD7797_MODE_MD1
#define	AD7797_MODE_OFFSET_SELF	AD7797_MODE_MD2
#define	AD7797_MODE_RES			AD7797_MODE_MD0 | AD7797_MODE_MD2
#define	AD7797_MODE_OFFSET_SYS	AD7797_MODE_MD1 | AD7797_MODE_MD2
#define	AD7797_MODE_FSCAL_SYS	AD7797_MODE_MD0 | AD7797_MODE_MD1 | AD7797_MODE_MD2
#define	AD7797_MODE_DEFAULT		AD7797_MODE_FS0 | AD7797_MODE_FS1 | AD7797_MODE_FS2 | AD7797_MODE_FS3

//config register bits
#define	AD7797_CFG_1			0x0710
#define	AD7797_CFG_BO			0x2000
#define	AD7797_CFG_UNI			0x1000
#define	AD7797_CFG_CH2			0x0004
#define	AD7797_CFG_CH1			0x0002
#define	AD7797_CFG_CH0			0x0001
//config register definitions
#define	AD7797_CFG_AINP_AINM	AD7797_CFG_1
#define	AD7797_CFG_AINM_AINM	AD7797_CFG_1 | AD7797_CFG_CH0 | AD7797_CFG_CH1
#define	AD7797_CFG_TEMP			AD7797_CFG_1 | AD7797_CFG_CH1 | AD7797_CFG_CH2
#define	AD7797_CFG_VDD			AD7797_CFG_1 | AD7797_CFG_CH0 | AD7797_CFG_CH1 | AD7797_CFG_CH2

//defaults
#define	AD7797_DEFAULT_GAIN		0x553300
#define	AD7797_DEFAULT_OFFSET	0x800000


void AD7797_SPI_start(void);
void AD7797_SPI_stop(void);
unsigned long AD7797_SPI_transfer(unsigned char command, unsigned char numbytes, unsigned long data);
unsigned char AD7797_init(void);
unsigned char AD7797_busy(unsigned char wait);
unsigned long AD7797_gain_systemcalibration(unsigned char wait);
unsigned long AD7797_offset_systemcalibration(unsigned char wait);
unsigned long AD7797_offset_selfcalibration(unsigned char wait);
unsigned long AD7797_gain_read(void);
void AD7797_gain_write(unsigned long gain);
unsigned long AD7797_offset_read(void);
void AD7797_offset_write(unsigned long offset);
unsigned long AD7797_convert(unsigned char chnum, unsigned char uni, unsigned char burnout, unsigned char wait);
unsigned long AD7797_convert_read(void);




#endif
