/**
 * ----------------------------------------------------------------------------------
 *
 * Copyright (C) 2011 runlevel3 GmbH Berlin
 *
 * ----------------------------------------------------------------------------------
 *
 * LICENSE:
 *
 * This file is part of USB-AD-Interface.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * ----------------------------------------------------------------------------------
 *
 * AVR Studio 4.18 / avr-gcc version 4.3.3
 *
 * ----------------------------------------------------------------------------------
 *
 * @copyright 2011 runlevel3 GmbH
 * @license   http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @link      http://www.runlevel3.de runlevel3
 **/


#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <util/crc16.h>

#include <avr/eeprom.h>

#include <config.h>
#include <ad7797.h>


_config *config = NULL;
unsigned char *config_raw;



void config_defaults(void)
{
   	config->offset = AD7797_DEFAULT_OFFSET;
   	config->gain = AD7797_DEFAULT_GAIN;
   	config->stat_loops = STAT_LOOPS_DEFAULT;
   	config->auto_convert = AUTO_CONVERT_DEFAULT;
   	config->auto_mode = AUTO_CONVERT_MODE;
   	config->auto_interval = AUTO_CONVERT_INTERVAL;
   	config->startup_offset = STARTUP_OFFSET_DEFAULT;
}

unsigned int get_config(unsigned int base_address)
{
	unsigned int cnt, crc16 = 0;

	if (!config) {
		config = calloc(1,sizeof(_config));
		config_raw = (unsigned char *)config;
	}

	eeprom_read_block(config_raw, (void *)base_address, sizeof(_config));
	for (cnt=0;cnt<sizeof(_config)-2;cnt++) {
		crc16 = _crc16_update(crc16, config_raw[cnt]);
	}
	if (config->crc16 != crc16) return 0;
	return 1;
}

unsigned int read_config(void)
{
	unsigned int rv = 1;

	if ( !get_config(CONFIG_BASE_ADDR)) {
		if ( !get_config(CONFIG_BASE_ADDR_ALT)) config_defaults();
		write_config();
		rv = 0;
	}

	return rv;
}

void write_config(void)
{
	unsigned int cnt, crc16 = 0;

	if (!config) return;

	for (cnt=0;cnt<sizeof(_config)-2;cnt++) {
		crc16 = _crc16_update(crc16, config_raw[cnt]);
	}
	config->crc16 = crc16;

	eeprom_update_block(config_raw, (void *)CONFIG_BASE_ADDR, sizeof(_config));
	eeprom_update_block(config_raw, (void *)CONFIG_BASE_ADDR_ALT, sizeof(_config));
}
