/**
 * ----------------------------------------------------------------------------------
 *
 * Copyright (C) 2011 runlevel3 GmbH Berlin
 *
 * ----------------------------------------------------------------------------------
 *
 * LICENSE:
 *
 * This file is part of USB-AD-Interface.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * ----------------------------------------------------------------------------------
 *
 * AVR Studio 4.18 / avr-gcc version 4.3.3
 *
 * ----------------------------------------------------------------------------------
 *
 * @copyright 2011 runlevel3 GmbH
 * @license   http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @link      http://www.runlevel3.de runlevel3
 **/


#include <avr/io.h>
#include <avr/interrupt.h>

#include <timer_x.h>
#include <config.h>
#include <ad7797.h>
#include <usb-commands.h>


//external values
extern volatile unsigned long ad_value, offset_new, gain_new;
extern volatile unsigned char ad_ready, timer_signals, convert_last, convert_new;


//global config record
extern _config *config;


volatile unsigned long convert_timer = 0, busy_timer = 0;


ISR (TIMER1_COMPA_vect, ISR_NOBLOCK)
{
	//AD converter busy -> decrement wait counters
	if (AD7797_busy(0)) {
		busy_timer++;
		if (convert_timer > 1) convert_timer--;
		if (busy_timer > 10000) {
			busy_timer = 0;
			AD7797_convert(convert_last & 7, (convert_last & 0x40) >> 6, (convert_last & 0x80) >> 7, 0);
		}
		return;
	}
	busy_timer = 0;

	//autoconvert ready -> read result
	if (convert_timer) {
		convert_timer--;
		if (!convert_timer) {
			ad_value = AD7797_convert_read();
			if (config->auto_convert) convert_timer = config->auto_interval;
			ad_ready = 1;
		}
	}

	//read result of calibration
	if (timer_signals & SIGNAL_READ_OFFSET) {
		timer_signals &= ~SIGNAL_READ_OFFSET;
		config->offset = AD7797_offset_read();
		//restart conversion
		timer_signals |= SIGNAL_CONVERT;
	}
	if (timer_signals & SIGNAL_READ_GAIN) {
		timer_signals &= ~SIGNAL_READ_GAIN;
		config->gain = AD7797_gain_read();
		//restart conversion
		timer_signals |= SIGNAL_CONVERT;
	}

	//start conversion
	if (timer_signals & SIGNAL_CONVERT) {
		timer_signals &= ~SIGNAL_CONVERT;
		AD7797_convert(convert_last & 7, (convert_last & 0x40) >> 6, (convert_last & 0x80) >> 7, 0);
		convert_timer = AUTO_CONVERT_INTERVAL - 100;
		return;
	}

	//send usb-commands to converter
	if (timer_signals & SIGNAL_WRITE_OFFSET) {
		timer_signals &= ~SIGNAL_WRITE_OFFSET;
		config->offset = offset_new;
		AD7797_offset_write(offset_new);
		return;
	}
	if (timer_signals & SIGNAL_WRITE_GAIN) {
		timer_signals &= ~SIGNAL_WRITE_GAIN;
		config->gain = gain_new;
		AD7797_gain_write(gain_new);
		return;
	}
	if (timer_signals & SIGNAL_OFFSET_SELFCALIBRATION) {
		timer_signals &= ~SIGNAL_OFFSET_SELFCALIBRATION;
		//initiate selfcalibration
		AD7797_offset_selfcalibration(0);
		//read offset after selfcalibration complete
		timer_signals |= SIGNAL_READ_OFFSET;
		return;
	}
	if (timer_signals & SIGNAL_OFFSET_SYSTEMCALIBRATION) {
		timer_signals &= ~SIGNAL_OFFSET_SYSTEMCALIBRATION;
		//initiate systemcalibration
		AD7797_offset_systemcalibration(0);
		//read offset after systemcalibration complete
		timer_signals |= SIGNAL_READ_OFFSET;
		return;
	}
	if (timer_signals & SIGNAL_GAIN_SYSTEMCALIBRATION) {
		timer_signals &= ~SIGNAL_GAIN_SYSTEMCALIBRATION;
		//initiate systemcalibration
		AD7797_gain_systemcalibration(0);
		//read gain after systemcalibration complete
		timer_signals |= SIGNAL_READ_GAIN;
		return;
	}
}

void Timer1_int_disable(void)
{
	TIMSK1 &= ~_BV(OCIE1A);
}

void Timer1_int_enable(void)
{
	TIMSK1 |= _BV(OCIE1A);
}

void Timer1_init(void)
{
	//COM1A, B, C = 0 (Compare Output Mode) = normal (output pin disconnected)
	//WGM = 4 (CTC - Clear Timer on Compare match)
	//CS = 1 (prescaler = 1)

	TCCR1B = _BV(WGM12) | _BV(CS10);	//0x09

	//divisor
	OCR1A = TIMER1_DIVISOR;

	Timer1_int_enable();
}
