/**
 * ----------------------------------------------------------------------------------
 *
 * Copyright (C) 2011 runlevel3 GmbH Berlin
 *
 * ----------------------------------------------------------------------------------
 *
 * LICENSE:
 *
 * This file is part of USB-AD-Interface.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * ----------------------------------------------------------------------------------
 *
 * AVR Studio 4.18 / avr-gcc version 4.3.3
 *
 * ----------------------------------------------------------------------------------
 *
 * @copyright 2011 runlevel3 GmbH
 * @license   http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @link      http://www.runlevel3.de runlevel3
 **/


#include <stdio.h>
#include <stdlib.h>

#include <uart-int.h>
#include <avr/io.h>
#include <avr/interrupt.h>


unsigned char  *sendbuff;
unsigned char  *recvbuff;
unsigned int sendbuff_inptr = 0;
unsigned int sendbuff_outptr = 0;
unsigned int recvbuff_inptr = 0;
unsigned int recvbuff_outptr = 0;
unsigned int sendbuffsize = 0;
unsigned int recvbuffsize = 0;


unsigned int USART_inbuffused(void) {
	int tmpsize;

	tmpsize = recvbuff_outptr - recvbuff_inptr;
	if (tmpsize < 0) tmpsize += recvbuffsize;
	return (unsigned int)tmpsize;
}

unsigned int USART_outbuffused(void) {
	int tmpsize;

	tmpsize = sendbuff_outptr - sendbuff_inptr;
	if (tmpsize < 0) tmpsize += sendbuffsize;
	return (unsigned int)tmpsize;
}

unsigned int USART_inbuffsize(void) {
	return recvbuffsize;
}

unsigned int USART_outbuffsize(void) {
	return sendbuffsize;
}

unsigned int USART_inbufffree(void) {
	return USART_inbuffsize() - USART_inbuffused();
}

unsigned int USART_outbufffree(void) {
	return USART_outbuffsize() - USART_outbuffused();
}

ISR (USART0_RX_vect) {
	recvbuff[recvbuff_inptr] = UDR0;
	recvbuff_inptr += 1;
	//enable global interrupts for other pendig requests
	sei();
	if (recvbuff_inptr == recvbuffsize) recvbuff_inptr = 0;
}

ISR (USART0_UDRE_vect) {
	if (sendbuff_outptr == sendbuff_inptr) {
		//keine Daten -> disable Data empty Interrupt
		UCSR0B &= ~(1<<UDRIE0);
		return;
	}
	UDR0 = sendbuff[sendbuff_outptr];
	sendbuff_outptr += 1;
	//enable global interrupts for other pendig requests
	sei();
	if (sendbuff_outptr == sendbuffsize) sendbuff_outptr = 0;
}

unsigned char USART_init(unsigned int baud, unsigned char dbaud, unsigned int _sendbuffsize, unsigned int _recvbuffsize) {
	sendbuffsize = _sendbuffsize ? _sendbuffsize : SENDBUFFSIZE_DEFAULT;
	recvbuffsize = _recvbuffsize ? _recvbuffsize : RECVBUFFSIZE_DEFAULT;
	if (!(sendbuff = malloc(sendbuffsize))) return 0;
	if (!(recvbuff = malloc(recvbuffsize))) return 0;
  
	UBRR0H = baud>>8 & 0xff;
	UBRR0L = baud & 0xff;
	if ( dbaud ) UCSR0A |= 1<<U2X0;
	UCSR0C = ( 3<<UCSZ00 );
	//Transmitter +  Reciever enable, enable receive Interrupt
	UCSR0B = ( 1<<RXEN0 | 1<<TXEN0 | 1<<RXCIE0 );
	return 1;
}

void USART_free(void) {
	//Transmitter +  Reciever disable, disable receive Interrupt
	UCSR0B = 0;
	free(sendbuff);
	free(recvbuff);
}

int USART_putchar(char usart_char, FILE *stream) {
	if (!USART_outbufffree()) return -1;
	sendbuff[sendbuff_inptr] = usart_char;
	sendbuff_inptr++;
	if (sendbuff_inptr == sendbuffsize) sendbuff_inptr = 0;
	UCSR0B |= 1<<UDRIE0;
	return 0;
}

int USART_getchar(FILE *stream) {
	char tmpchar;
	
	if (!USART_inbuffused()) return _FDEV_EOF;
	tmpchar = recvbuff[recvbuff_outptr];
	recvbuff_outptr++;
	if (recvbuff_outptr == recvbuffsize) recvbuff_outptr = 0;
	return (int)tmpchar;
}

unsigned int USART_transmit(char *usart_text) {
	unsigned int cnt=0;
	
	while (usart_text[cnt] != '\0')	{
		if (USART_putchar(usart_text[cnt], NULL)) return cnt;
		cnt++;
	}
	return cnt;
}

unsigned int USART_receive(char *usart_text, unsigned int max) {
	unsigned int cnt=0;
	
	while ((USART_inbuffused > 0) && (cnt < max)) {
		usart_text[cnt] = USART_getchar(NULL);
		cnt++;
	}
	return cnt;
}
